;;; ada-xref.el --- Dummy file to hide obsolete bundled version  -*- lexical-binding:t -*-
;;
;; Copyright (C) 2020  Free Software Foundation, Inc.
;; see xref-ada.el for Emacs 25 xref minor mode for ada-mode
;;
;; Keep this file until drop support for Emacs 26; core ada-mode deleted in Emacs 27.

(require 'ada-mode)
(provide 'ada-xref)
(message "'ada-xref' is obsolete; use 'ada-mode' instead")
