;; dummy file to hide obsolete bundled version
;;
;; Copyright (C) 2020  Free Software Foundation, Inc.
;;
;; Keep this file until drop support for Emacs 26; core ada-mode deleted in Emacs 27.

(require 'ada-mode)
(provide 'ada-prj)
(message "'ada-prj' is obsolete; use 'ada-mode' instead")
